"""
Copyright (c) 2014-2017, Bitnine Inc.
Copyright (c) 2022, Vinzenz Feenstra

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re
from typing import Optional

from psycopg import abc
from psycopg.adapt import Dumper, Loader
from psycopg.errors import InterfaceError

from psycopg3_agensgraph._oids import _GRAPHID_OID

_pattern = re.compile(rb"(\d+)\.(\d+)")


class GraphId(object):
    def __init__(self, gid):
        self.gid = gid

    def getId(self):
        return self.gid

    def __eq__(self, other):
        if isinstance(self, other.__class__):
            return self.gid == other.gid
        return False

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, self)

    def __str__(self):
        return "{0}.{1}".format(*self.gid)


class GraphIdLoader(Loader):
    def __init__(
        self,
        oid: int = _GRAPHID_OID,
        context: Optional[abc.AdaptContext] = None,
    ):
        super().__init__(oid, context)

    def load(self, value):
        if value is None:
            return None
        value = value if isinstance(value, (bytes, memoryview)) else value.encode()
        m = _pattern.match(value)
        if m:
            labid = int(m.group(1))
            locid = int(m.group(2))
            gid = (labid, locid)
            return GraphId(gid)
        else:
            raise InterfaceError("bad graphid representation: %s" % value)


class GraphIdDumper(Dumper):
    oid = _GRAPHID_OID

    def dump(self, graphid):
        return str(graphid).encode()
